@@ -32,60 +32,56 @@ NNS.dep = function(x,
                    asym = FALSE,
                    p.value = FALSE,
                    print.map = FALSE){
-
-
-
+  
+  
+  
   if(any(class(x)%in%c("tbl","data.table")) && !is.null(y)) x <- as.vector(unlist(x))
   if(any(class(y)%in%c("tbl","data.table"))) y <- as.vector(unlist(y))
-
+  
   if(sum(is.na(x)) > 0) stop("You have some missing values, please address.")
-
+  
   if(p.value){
     y_p <- replicate(100, sample.int(length(y)))
     x <- cbind(x, y, matrix(y[y_p], ncol = dim(y_p)[2], byrow = F))
     y <- NULL
   }
-
+  
   if(!is.null(y)){
     x <- as.numeric(x)
     l <- length(x)
     
     y <- as.numeric(y)
-    obs <- max(10, l/5)
+    obs <- max(8, l/8)
     
     # Define segments
-    if(print.map) PART_xy <- suppressWarnings(NNS.part(x, y, order = NULL, obs.req = obs, min.obs.stop = TRUE, type = "XONLY", Voronoi = TRUE)) else PART_xy <- suppressWarnings(NNS.part(x, y, order = NULL, obs.req = obs, min.obs.stop = TRUE, type = "XONLY", Voronoi = FALSE))
+    if(print.map) PART_xy <- suppressWarnings(NNS.part(x, y, order = NULL, obs.req = obs, min.obs.stop = FALSE, type = NULL, Voronoi = TRUE)) else PART_xy <- suppressWarnings(NNS.part(x, y, order = NULL, obs.req = obs, min.obs.stop = FALSE, type = "XONLY", Voronoi = FALSE))
     
-    PART_yx <- suppressWarnings(NNS.part(y, x, order = NULL, obs.req = obs, min.obs.stop = TRUE, type = "XONLY", Voronoi = FALSE))
+    PART_yx <- suppressWarnings(NNS.part(y, x, order = NULL, obs.req = obs, min.obs.stop = FALSE, type = "XONLY", Voronoi = FALSE))
     
     if(dim(PART_xy$regression.points)[1]==0) return(list("Correlation" = 0, "Dependence" = 0))
     
     PART_xy <- PART_xy$dt
     PART_xy <- PART_xy[complete.cases(PART_xy),]
     
-    PART_xy[, weights_xy := .N/l, by = prior.quadrant]
-    weights_xy <- PART_xy[, weights_xy[1], by = prior.quadrant]$V1
+    PART_xy[, weights_xy := .N/l, by = quadrant]
+    weights_xy <- PART_xy[, weights_xy[1], by = quadrant]$V1
     
     PART_yx <- PART_yx$dt
     PART_yx <- PART_yx[complete.cases(PART_yx),]
     
-    PART_yx[, weights_yx := .N/l, by = prior.quadrant]
-    weights_yx <- PART_yx[, weights_yx[1], by = prior.quadrant]$V1
-    
-    
-    ll <- expression(max(.N, 8))
+    PART_yx[, weights_yx := .N/l, by = quadrant]
+    weights_yx <- PART_yx[, weights_yx[1], by = quadrant]$V1
     
     
     dep_fn = function(x, y){
       NNS::NNS.copula(cbind(x, y)) * sign(cov(x,y))
     }
     
+    res_xy <- suppressWarnings(tryCatch(PART_xy[,  dep_fn(x, y), by = quadrant],
+                                        error = function(e)PART_xy[,  dep_fn(x, y), by = prior.quadrant]))
     
-    res_xy <- suppressWarnings(tryCatch(PART_xy[1:eval(ll),  dep_fn(x, y), by = prior.quadrant],
-                                        error = function(e) dep_fn(x, y)))
-    
-    res_yx <- suppressWarnings(tryCatch(PART_yx[1:eval(ll),  dep_fn(y, x), by = prior.quadrant],
-                                        error = function(e) dep_fn(y, x)))
+    res_yx <- suppressWarnings(tryCatch(PART_yx[,  dep_fn(y, x), by = quadrant],
+                                        error = function(e) PART_yx[,  dep_fn(y, x), by = prior.quadrant]))
     
     if(sum(is.na(res_xy))>0) res_xy[is.na(res_xy)] <- dep_fn(x, y)
     if(is.null(ncol(res_xy))) res_xy <- cbind(res_xy, res_xy)
@@ -128,14 +124,14 @@ NNS.dep = function(x,
   } else {
     if(p.value){
       original.par <- par(no.readonly = TRUE)
-
+      
       nns.mc <- apply(x, 2, function(g) NNS.dep(x[,1], g))
-
+      
       ## Store results
       cors <- unlist(lapply(nns.mc, "[[", 1))
       deps <- unlist(lapply(nns.mc, "[[", 2))
-
-
+      
+      
       cor_lower_CI <- LPM.VaR(.025, 0, cors[-c(1,2)])
       cor_upper_CI <- UPM.VaR(.025, 0, cors[-c(1,2)])
       dep_lower_CI <- LPM.VaR(.025, 0, deps[-c(1,2)])
@@ -155,7 +151,7 @@ NNS.dep = function(x,
         abline(v =  dep_upper_CI , col = "red", lwd = 2, lty = 3)
         par(mfrow = original.par)
       }
-
+      
       return(list("Correlation" = as.numeric((cors)[2]),
                   "Correlation p.value" = min(LPM(0, cors[2], cors[-c(1,2)]),
                                               UPM(0, cors[2], cors[-c(1,2)])),
@@ -166,5 +162,5 @@ NNS.dep = function(x,
                   "Dependence 95% CIs" = c(dep_lower_CI, dep_upper_CI)))
     } else return(NNS.dep.matrix(x, asym = asym))
   }
-
-}
+  
+}
\ No newline at end of file
