@@ -64,27 +64,26 @@ NNS.reg(iris[ , 1 : 4], iris[ , 5], type = "CLASS", point.est = iris[1 : 10, 1 :
 
 ## ----stackevalres, eval = FALSE-----------------------------------------------
 #  Folds Remaining = 0
-#  Current NNS.reg(... , threshold = 0.935 ) MAX Iterations Remaining = 2
-#  Current NNS.reg(... , threshold = 0.795 ) MAX Iterations Remaining = 1
-#  Current NNS.reg(... , threshold = 0.44 ) MAX Iterations Remaining = 0
+#  Current NNS.reg(... , threshold = 0.8 ) MAX Iterations Remaining = 1
+#  Current NNS.reg(... , threshold = 0.405 ) MAX Iterations Remaining = 0
 #  Current NNS.reg(... , n.best = 1 ) MAX Iterations Remaining = 12
 #  Current NNS.reg(... , n.best = 2 ) MAX Iterations Remaining = 11
 #  Current NNS.reg(... , n.best = 3 ) MAX Iterations Remaining = 10
 #  Current NNS.reg(... , n.best = 4 ) MAX Iterations Remaining = 9
 #  $OBJfn.reg
-#  [1] 1
+#  [1] 0.9933333
 #  
 #  $NNS.reg.n.best
-#  [1] 4
+#  [1] 1
 #  
 #  $probability.threshold
 #  [1] 0.43875
 #  
 #  $OBJfn.dim.red
-#  [1] 0.9666667
+#  [1] 0.96
 #  
 #  $NNS.dim.red.threshold
-#  [1] 0.935
+#  [1] 0.8
 #  
 #  $reg
 #   [1] 1 1 1 1 1 1 1 1 1 1
@@ -126,6 +125,9 @@ NNS.reg(cbind(x, x), y,
         residual.plot = TRUE,  
         ncores = 1, confidence.interval = .95)
 
+## ----smooth, fig.width=5,fig.height=3,fig.align = "center",results='hide'-----
+NNS.reg(x, y, smooth = T)
+
 ## ----threads, echo = FALSE----------------------------------------------------
 Sys.setenv("OMP_THREAD_LIMIT" = "")
 
