@@ -52,8 +52,8 @@ NNS.dep(y, x, asym = TRUE)
 
 ## ----dependence,fig.width=5,fig.height=3,fig.align = "center"-----------------
 set.seed(123)
-df <- data.frame(x = runif(10000, -1, 1), y = runif(10000, -1, 1))
-df <- subset(df, (x ^ 2 + y ^ 2 <= 1 & x ^ 2 + y ^ 2 >= 0.95))
+df = data.frame(x = runif(10000, -1, 1), y = runif(10000, -1, 1))
+df = subset(df, (x ^ 2 + y ^ 2 <= 1 & x ^ 2 + y ^ 2 >= 0.95))
 
 ## ----circle1,fig.width=5,fig.height=3,fig.align = "center", results='hide', echo=FALSE----
 NNS.part(df$x, df$y, Voronoi = TRUE, order = 3, obs.req = 0)
@@ -64,7 +64,7 @@ NNS.dep(df$x, df$y)
 ## ----permutations-------------------------------------------------------------
 ## p-values for [NNS.dep]
 set.seed(123)
-x <- seq(-5, 5, .1); y <- x^2 + rnorm(length(x))
+x = seq(-5, 5, .1); y = x^2 + rnorm(length(x))
 
 ## ----perm1,fig.width=5,fig.height=3,fig.align = "center", results='hide', echo=FALSE----
 NNS.part(x, y, Voronoi = TRUE, order = 3)
@@ -74,7 +74,7 @@ NNS.dep(x, y, p.value = TRUE, print.map = TRUE)
 
 ## ----multi, warning=FALSE-----------------------------------------------------
 set.seed(123)
-x <- rnorm(1000); y <- rnorm(1000); z <- rnorm(1000)
+x = rnorm(1000); y = rnorm(1000); z = rnorm(1000)
 NNS.copula(cbind(x, y, z), plot = TRUE, independence.overlay = TRUE)
 
 ## ----threads, echo = FALSE----------------------------------------------------
